@@ -209,7 +209,7 @@ struct bcm2835_host {
 	/*end of DMA part*/
 
 	int				max_delay;	/* maximum length of time spent waiting */
-	struct timeval			stop_time;	/* when the last stop was issued */
+	struct timespec64		stop_time;	/* when the last stop was issued */
 	u32				delay_after_stop; /* minimum time between stop and subsequent data transfer */
 	u32				delay_after_this_stop; /* minimum time between this stop and subsequent data transfer */
 	u32				user_overclock_50; /* User's preferred frequency to use when 50MHz is requested (in MHz) */
@@ -243,12 +243,12 @@ static void __iomem *timer_base;
 static void log_init(struct device *dev, u32 bus_to_phys)
 {
 	spin_lock_init(&log_lock);
-	sdhost_log_buf = dma_zalloc_coherent(dev, LOG_SIZE, &sdhost_log_addr,
+	sdhost_log_buf = dma_alloc_coherent(dev, LOG_SIZE, &sdhost_log_addr,
 					     GFP_KERNEL);
 	if (sdhost_log_buf) {
 		pr_info("sdhost: log_buf @ %p (%llx)\n",
 			sdhost_log_buf, (u64)sdhost_log_addr);
-		timer_base = ioremap_nocache(bus_to_phys + 0x7e003000, SZ_4K);
+		timer_base = ioremap(bus_to_phys + 0x7e003000, SZ_4K);
 		if (!timer_base)
 			pr_err("sdhost: failed to remap timer\n");
 	}
@@ -441,7 +441,6 @@ static void bcm2835_sdhost_reset_internal(struct bcm2835_host *host)
 	host->sectors = 0;
 	bcm2835_sdhost_write(host, host->hcfg, SDHCFG);
 	bcm2835_sdhost_write(host, SDCDIV_MAX_CDIV, SDCDIV);
-	mmiowb();
 }
 
 static void bcm2835_sdhost_reset(struct mmc_host *mmc)
@@ -980,7 +979,7 @@ bool bcm2835_sdhost_send_command(struct bcm2835_host *host,
 	delay = (10000 - timeout)/100;
 	if (delay > host->max_delay) {
 		host->max_delay = delay;
-		pr_warning("%s: controller hung for %d ms\n",
+		pr_warn("%s: controller hung for %d ms\n",
 			   mmc_hostname(host->mmc),
 			   host->max_delay);
 	}
@@ -1028,14 +1027,15 @@ bool bcm2835_sdhost_send_command(struct bcm2835_host *host,
 	if (cmd->data) {
 		log_event("CMDD", cmd->data->blocks, cmd->data->blksz);
 		if (host->delay_after_this_stop) {
-			struct timeval now;
+			struct timespec64 now;
 			int time_since_stop;
-			do_gettimeofday(&now);
-			time_since_stop = (now.tv_sec - host->stop_time.tv_sec);
+
+			ktime_get_real_ts64(&now);
+			time_since_stop = now.tv_sec - host->stop_time.tv_sec;
 			if (time_since_stop < 2) {
 				/* Possibly less than one second */
 				time_since_stop = time_since_stop * 1000000 +
-					(now.tv_usec - host->stop_time.tv_usec);
+					(now.tv_nsec - host->stop_time.tv_nsec)/1000;
 				if (time_since_stop <
 				    host->delay_after_this_stop)
 					udelay(host->delay_after_this_stop -
@@ -1137,7 +1137,7 @@ static void bcm2835_sdhost_transfer_complete(struct bcm2835_host *host)
 				bcm2835_sdhost_finish_command(host, NULL);
 
 			if (host->delay_after_this_stop)
-				do_gettimeofday(&host->stop_time);
+				ktime_get_real_ts64(&host->stop_time);
 		}
 	} else {
 		bcm2835_sdhost_wait_transfer_complete(host);
@@ -1154,7 +1154,7 @@ static void bcm2835_sdhost_finish_command(struct bcm2835_host *host,
 	u32 sdcmd;
 	u32 retries;
 #ifdef DEBUG
-	struct timeval before, after;
+	struct timespec64 before, after;
 	int timediff = 0;
 #endif
 
@@ -1168,7 +1168,7 @@ static void bcm2835_sdhost_finish_command(struct bcm2835_host *host,
 	retries = host->cmd_quick_poll_retries;
 	if (!retries) {
 		/* Work out how many polls take 1us by timing 10us */
-		struct timeval start, now;
+		struct timespec64 start, now;
 		int us_diff;
 
 		retries = 1;
@@ -1177,16 +1177,16 @@ static void bcm2835_sdhost_finish_command(struct bcm2835_host *host,
 
 			retries *= 2;
 
-			do_gettimeofday(&start);
+			ktime_get_real_ts64(&start);
 
 			for (i = 0; i < retries; i++) {
 				cpu_relax();
 				sdcmd = bcm2835_sdhost_read(host, SDCMD);
 			}
 
-			do_gettimeofday(&now);
+			ktime_get_real_ts64(&now);
 			us_diff = (now.tv_sec - start.tv_sec) * 1000000 +
-				(now.tv_usec - start.tv_usec);
+				(now.tv_nsec - start.tv_nsec)/1000;
 		} while (us_diff < 10);
 
 		host->cmd_quick_poll_retries = ((retries * us_diff + 9)*CMD_DALLY_US)/10 + 1;
@@ -1346,7 +1346,6 @@ static void bcm2835_sdhost_timeout(struct timer_list *t)
 		}
 	}
 
-	mmiowb();
 	spin_unlock_irqrestore(&host->lock, flags);
 }
 
@@ -1515,8 +1514,6 @@ static irqreturn_t bcm2835_sdhost_irq(int irq, void *dev_id)
 		result = IRQ_HANDLED;
 	}
 
-	mmiowb();
-
 	log_event("IRQ>", bcm2835_sdhost_read(host, SDHSTS), 0);
 	spin_unlock(&host->lock);
 
@@ -1575,7 +1572,6 @@ void bcm2835_sdhost_set_clock(struct bcm2835_host *host, unsigned int clock)
 			 */
 			host->cdiv = SDCDIV_MAX_CDIV;
 			bcm2835_sdhost_write(host, host->cdiv, SDCDIV);
-			mmiowb();
 			spin_unlock_irqrestore(&host->lock, flags);
 			return;
 		}
@@ -1639,7 +1635,6 @@ void bcm2835_sdhost_set_clock(struct bcm2835_host *host, unsigned int clock)
 	host->clock = input_clock;
 	host->reset_clock = 0;
 
-	mmiowb();
 	spin_unlock_irqrestore(&host->lock, flags);
 }
 
@@ -1714,7 +1709,6 @@ static void bcm2835_sdhost_request(struct mmc_host *mmc, struct mmc_request *mrq
 		}
 		mrq->cmd->error = -EILSEQ;
 		tasklet_schedule(&host->finish_tasklet);
-		mmiowb();
 		spin_unlock_irqrestore(&host->lock, flags);
 		return;
 	}
@@ -1737,7 +1731,6 @@ static void bcm2835_sdhost_request(struct mmc_host *mmc, struct mmc_request *mrq
 
 	log_event("CMD ", mrq->cmd->opcode,
 		   mrq->data ? (u32)mrq->data->blksz : 0);
-	mmiowb();
 
 	log_event("REQ>", mrq, 0);
 	spin_unlock_irqrestore(&host->lock, flags);
@@ -1772,8 +1765,6 @@ static void bcm2835_sdhost_set_ios(struct mmc_host *mmc, struct mmc_ios *ios)
 
 	bcm2835_sdhost_write(host, host->hcfg, SDHCFG);
 
-	mmiowb();
-
 	spin_unlock_irqrestore(&host->lock, flags);
 
 	if (!ios->clock || ios->clock != host->clock)
@@ -1808,8 +1799,6 @@ static void bcm2835_sdhost_cmd_wait_work(struct work_struct *work)
 
 	bcm2835_sdhost_finish_command(host, &flags);
 
-	mmiowb();
-
 	log_event("CWK>", host->cmd, 0);
 
 	spin_unlock_irqrestore(&host->lock, flags);
@@ -1862,8 +1851,6 @@ static void bcm2835_sdhost_tasklet_finish(unsigned long param)
 	host->cmd = NULL;
 	host->data = NULL;
 
-	mmiowb();
-
 	host->dma_desc = NULL;
 	terminate_chan = host->dma_chan;
 	host->dma_chan = NULL;
@@ -1918,7 +1905,7 @@ int bcm2835_sdhost_add_host(struct bcm2835_host *host)
 	/* host controller capabilities */
 	mmc->caps |=
 		MMC_CAP_SD_HIGHSPEED | MMC_CAP_MMC_HIGHSPEED |
-		MMC_CAP_NEEDS_POLL | MMC_CAP_HW_RESET | MMC_CAP_ERASE |
+		MMC_CAP_NEEDS_POLL | MMC_CAP_HW_RESET |
 		((ALLOW_CMD23_READ|ALLOW_CMD23_WRITE) * MMC_CAP_CMD23);
 
 	spin_lock_init(&host->lock);
@@ -1995,7 +1982,6 @@ int bcm2835_sdhost_add_host(struct bcm2835_host *host)
 		goto untasklet;
 	}
 
-	mmiowb();
 	mmc_add_host(mmc);
 
 	pio_limit_string[0] = '\0';
