@@ -55,12 +55,12 @@ struct rpi_cirrus_priv {
 /* helper functions */
 static inline struct snd_soc_pcm_runtime *get_wm5102_runtime(
 	struct snd_soc_card *card) {
-	return snd_soc_get_pcm_runtime(card, card->dai_link[DAI_WM5102].name);
+	return snd_soc_get_pcm_runtime(card, &card->dai_link[DAI_WM5102]);
 }
 
 static inline struct snd_soc_pcm_runtime *get_wm8804_runtime(
 	struct snd_soc_card *card) {
-	return snd_soc_get_pcm_runtime(card, card->dai_link[DAI_WM8804].name);
+	return snd_soc_get_pcm_runtime(card, &card->dai_link[DAI_WM8804]);
 }
 
 
@@ -187,7 +187,8 @@ static int rpi_cirrus_spdif_playback_put(struct snd_kcontrol *kcontrol,
 	struct snd_ctl_elem_value *ucontrol)
 {
 	struct snd_soc_card *card = snd_kcontrol_chip(kcontrol);
-	struct snd_soc_codec *wm8804_codec = get_wm8804_runtime(card)->codec;
+	struct snd_soc_component *wm8804_component =
+		asoc_rtd_to_codec(get_wm8804_runtime(card), 0)->component;
 	struct rpi_cirrus_priv *priv = snd_soc_card_get_drvdata(card);
 	unsigned char *stat = priv->iec958_status;
 	unsigned char *ctrl_stat = ucontrol->value.iec958.status;
@@ -199,7 +200,7 @@ static int rpi_cirrus_spdif_playback_put(struct snd_kcontrol *kcontrol,
 		if ((ctrl_stat[i] & mask) != (stat[i] & mask)) {
 			changed = 1;
 			stat[i] = ctrl_stat[i] & mask;
-			snd_soc_update_bits(wm8804_codec,
+			snd_soc_component_update_bits(wm8804_component,
 				WM8804_SPDTX1 + i, mask, stat[i]);
 		}
 	}
@@ -222,14 +223,16 @@ static int rpi_cirrus_spdif_capture_get(struct snd_kcontrol *kcontrol,
 	struct snd_ctl_elem_value *ucontrol)
 {
 	struct snd_soc_card *card = snd_kcontrol_chip(kcontrol);
-	struct snd_soc_codec *wm8804_codec = get_wm8804_runtime(card)->codec;
-	unsigned int mask;
+	struct snd_soc_component *wm8804_component =
+		asoc_rtd_to_codec(get_wm8804_runtime(card), 0)->component;
+	unsigned int val, mask;
 	int i;
 
 	for (i = 0; i < 4; i++) {
+		val = snd_soc_component_read(wm8804_component,
+			WM8804_RXCHAN1 + i);
 		mask = (i == 3) ? 0x3f : 0xff;
-		ucontrol->value.iec958.status[i] =
-			snd_soc_read(wm8804_codec, WM8804_RXCHAN1 + i) & mask;
+		ucontrol->value.iec958.status[i] = val & mask;
 	}
 
 	return 0;
@@ -252,13 +255,18 @@ static int rpi_cirrus_spdif_status_flag_get(struct snd_kcontrol *kcontrol,
 	struct snd_ctl_elem_value *ucontrol)
 {
 	struct snd_soc_card *card = snd_kcontrol_chip(kcontrol);
-	struct snd_soc_codec *wm8804_codec = get_wm8804_runtime(card)->codec;
+	struct snd_soc_component *wm8804_component =
+		asoc_rtd_to_codec(get_wm8804_runtime(card), 0)->component;
 
 	unsigned int bit = kcontrol->private_value & 0xff;
 	unsigned int reg = (kcontrol->private_value >> 8) & 0xff;
 	unsigned int invert = (kcontrol->private_value >> 16) & 0xff;
+	unsigned int val;
+	bool flag;
 
-	bool flag = snd_soc_read(wm8804_codec, reg) & (1 << bit);
+	val = snd_soc_component_read(wm8804_component, reg);
+
+	flag = val & (1 << bit);
 
 	ucontrol->value.integer.value[0] = invert ? !flag : flag;
 
@@ -292,10 +300,13 @@ static int rpi_cirrus_recovered_frequency_get(struct snd_kcontrol *kcontrol,
 	struct snd_ctl_elem_value *ucontrol)
 {
 	struct snd_soc_card *card = snd_kcontrol_chip(kcontrol);
-	struct snd_soc_codec *wm8804_codec = get_wm8804_runtime(card)->codec;
+	struct snd_soc_component *wm8804_component =
+		asoc_rtd_to_codec(get_wm8804_runtime(card), 0)->component;
+	unsigned int val;
 
-	ucontrol->value.enumerated.item[0] =
-		(snd_soc_read(wm8804_codec, WM8804_SPDSTAT) >> 4) & 0x03;
+	val = snd_soc_component_read(wm8804_component, WM8804_SPDSTAT);
+
+	ucontrol->value.enumerated.item[0] = (val >> 4) & 0x03;
 	return 0;
 }
 
@@ -416,13 +427,13 @@ const struct snd_soc_dapm_route rpi_cirrus_dapm_routes[] = {
 };
 
 static int rpi_cirrus_clear_flls(struct snd_soc_card *card,
-	struct snd_soc_codec *wm5102_codec) {
+	struct snd_soc_component *wm5102_component) {
 
 	int ret1, ret2;
 
-	ret1 = snd_soc_codec_set_pll(wm5102_codec,
+	ret1 = snd_soc_component_set_pll(wm5102_component,
 		WM5102_FLL1, ARIZONA_FLL_SRC_NONE, 0, 0);
-	ret2 = snd_soc_codec_set_pll(wm5102_codec,
+	ret2 = snd_soc_component_set_pll(wm5102_component,
 		WM5102_FLL1_REFCLK, ARIZONA_FLL_SRC_NONE, 0, 0);
 
 	if (ret1) {
@@ -439,9 +450,9 @@ static int rpi_cirrus_clear_flls(struct snd_soc_card *card,
 }
 
 static int rpi_cirrus_set_fll(struct snd_soc_card *card,
-	struct snd_soc_codec *wm5102_codec, unsigned int clk_freq)
+	struct snd_soc_component *wm5102_component, unsigned int clk_freq)
 {
-	int ret = snd_soc_codec_set_pll(wm5102_codec,
+	int ret = snd_soc_component_set_pll(wm5102_component,
 		WM5102_FLL1,
 		ARIZONA_CLK_SRC_MCLK1,
 		WM8804_CLKOUT_HZ,
@@ -455,10 +466,10 @@ static int rpi_cirrus_set_fll(struct snd_soc_card *card,
 }
 
 static int rpi_cirrus_set_fll_refclk(struct snd_soc_card *card,
-	struct snd_soc_codec *wm5102_codec,
+	struct snd_soc_component *wm5102_component,
 	unsigned int clk_freq, unsigned int aif2_freq)
 {
-	int ret = snd_soc_codec_set_pll(wm5102_codec,
+	int ret = snd_soc_component_set_pll(wm5102_component,
 		WM5102_FLL1_REFCLK,
 		ARIZONA_CLK_SRC_MCLK1,
 		WM8804_CLKOUT_HZ,
@@ -470,7 +481,7 @@ static int rpi_cirrus_set_fll_refclk(struct snd_soc_card *card,
 		return ret;
 	}
 
-	ret = snd_soc_codec_set_pll(wm5102_codec,
+	ret = snd_soc_component_set_pll(wm5102_component,
 		WM5102_FLL1,
 		ARIZONA_CLK_SRC_AIF2BCLK,
 		aif2_freq, clk_freq);
@@ -488,7 +499,8 @@ static int rpi_cirrus_spdif_rx_enable_event(struct snd_soc_dapm_widget *w,
 {
 	struct snd_soc_card *card = w->dapm->card;
 	struct rpi_cirrus_priv *priv = snd_soc_card_get_drvdata(card);
-	struct snd_soc_codec *wm5102_codec = get_wm5102_runtime(card)->codec;
+	struct snd_soc_component *wm5102_component =
+		asoc_rtd_to_codec(get_wm5102_runtime(card), 0)->component;
 
 	unsigned int clk_freq, aif2_freq;
 	int ret = 0;
@@ -506,13 +518,13 @@ static int rpi_cirrus_spdif_rx_enable_event(struct snd_soc_dapm_widget *w,
 			"spdif_rx: changing FLL1 to use Ref Clock clk: %d spdif: %d\n",
 			clk_freq, aif2_freq);
 
-		ret = rpi_cirrus_clear_flls(card, wm5102_codec);
+		ret = rpi_cirrus_clear_flls(card, wm5102_component);
 		if (ret) {
 			dev_err(card->dev, "spdif_rx: failed to clear FLLs\n");
 			goto out;
 		}
 
-		ret = rpi_cirrus_set_fll_refclk(card, wm5102_codec,
+		ret = rpi_cirrus_set_fll_refclk(card, wm5102_component,
 			clk_freq, aif2_freq);
 
 		if (ret) {
@@ -545,12 +557,13 @@ static int rpi_cirrus_set_bias_level(struct snd_soc_card *card,
 {
 	struct rpi_cirrus_priv *priv = snd_soc_card_get_drvdata(card);
 	struct snd_soc_pcm_runtime *wm5102_runtime = get_wm5102_runtime(card);
-	struct snd_soc_codec *wm5102_codec = wm5102_runtime->codec;
+	struct snd_soc_component *wm5102_component =
+		asoc_rtd_to_codec(wm5102_runtime, 0)->component;
 
 	int ret = 0;
 	unsigned int clk_freq;
 
-	if (dapm->dev != wm5102_runtime->codec_dai->dev)
+	if (dapm->dev != asoc_rtd_to_codec(wm5102_runtime, 0)->dev)
 		return 0;
 
 	switch (level) {
@@ -567,7 +580,8 @@ static int rpi_cirrus_set_bias_level(struct snd_soc_card *card,
 				"set_bias: changing FLL1 from %d to %d\n",
 				priv->fll1_freq, clk_freq);
 
-			ret = rpi_cirrus_set_fll(card, wm5102_codec, clk_freq);
+			ret = rpi_cirrus_set_fll(card,
+				wm5102_component, clk_freq);
 			if (ret)
 				dev_err(card->dev,
 					"set_bias: Failed to set FLL1\n");
@@ -589,9 +603,10 @@ static int rpi_cirrus_set_bias_level_post(struct snd_soc_card *card,
 {
 	struct rpi_cirrus_priv *priv = snd_soc_card_get_drvdata(card);
 	struct snd_soc_pcm_runtime *wm5102_runtime = get_wm5102_runtime(card);
-	struct snd_soc_codec *wm5102_codec = wm5102_runtime->codec;
+	struct snd_soc_component *wm5102_component =
+		asoc_rtd_to_codec(wm5102_runtime, 0)->component;
 
-	if (dapm->dev != wm5102_runtime->codec_dai->dev)
+	if (dapm->dev != asoc_rtd_to_codec(wm5102_runtime, 0)->dev)
 		return 0;
 
 	switch (level) {
@@ -602,7 +617,7 @@ static int rpi_cirrus_set_bias_level_post(struct snd_soc_card *card,
 			"set_bias_post: changing FLL1 from %d to off\n",
 				priv->fll1_freq);
 
-		if (rpi_cirrus_clear_flls(card, wm5102_codec))
+		if (rpi_cirrus_clear_flls(card, wm5102_component))
 			dev_err(card->dev,
 				"set_bias_post: failed to clear FLLs\n");
 		else
@@ -683,9 +698,9 @@ static int rpi_cirrus_hw_params(struct snd_pcm_substream *substream,
 	struct snd_soc_pcm_runtime *rtd = substream->private_data;
 	struct snd_soc_card *card = rtd->card;
 	struct rpi_cirrus_priv *priv = snd_soc_card_get_drvdata(card);
-	struct snd_soc_dai *bcm_i2s_dai = rtd->cpu_dai;
-	struct snd_soc_codec *wm5102_codec = rtd->codec;
-	struct snd_soc_dai *wm8804_dai = get_wm8804_runtime(card)->codec_dai;
+	struct snd_soc_dai *bcm_i2s_dai = asoc_rtd_to_cpu(rtd, 0);
+	struct snd_soc_component *wm5102_component = asoc_rtd_to_codec(rtd, 0)->component;
+	struct snd_soc_dai *wm8804_dai = asoc_rtd_to_codec(get_wm8804_runtime(card), 0);
 
 	int ret;
 
@@ -704,7 +719,7 @@ static int rpi_cirrus_hw_params(struct snd_pcm_substream *substream,
 		goto out;
 	}
 
-	ret = snd_soc_dai_set_tdm_slot(rtd->codec_dai, 0x03, 0x03, 2, width);
+	ret = snd_soc_dai_set_tdm_slot(asoc_rtd_to_codec(rtd, 0), 0x03, 0x03, 2, width);
 	if (ret) {
 		dev_err(card->dev, "set_tdm_slot failed: %d\n", ret);
 		goto out;
@@ -717,7 +732,7 @@ static int rpi_cirrus_hw_params(struct snd_pcm_substream *substream,
 			goto out;
 	}
 
-	ret = snd_soc_codec_set_sysclk(wm5102_codec,
+	ret = snd_soc_component_set_sysclk(wm5102_component,
 		ARIZONA_CLK_SYSCLK,
 		ARIZONA_CLK_SRC_FLL1,
 		clk_freq,
@@ -732,12 +747,12 @@ static int rpi_cirrus_hw_params(struct snd_pcm_substream *substream,
 			"hw_params: changing FLL1 from %d to %d\n",
 			priv->fll1_freq, clk_freq);
 
-		if (rpi_cirrus_clear_flls(card, wm5102_codec)) {
+		if (rpi_cirrus_clear_flls(card, wm5102_component)) {
 			dev_err(card->dev, "hw_params: failed to clear FLLs\n");
 			goto out;
 		}
 
-		if (rpi_cirrus_set_fll(card, wm5102_codec, clk_freq)) {
+		if (rpi_cirrus_set_fll(card, wm5102_component, clk_freq)) {
 			dev_err(card->dev, "hw_params: failed to set FLL\n");
 			goto out;
 		}
@@ -762,7 +777,7 @@ static int rpi_cirrus_hw_free(struct snd_pcm_substream *substream)
 	struct snd_soc_pcm_runtime *rtd = substream->private_data;
 	struct snd_soc_card *card = rtd->card;
 	struct rpi_cirrus_priv *priv = snd_soc_card_get_drvdata(card);
-	struct snd_soc_codec *wm5102_codec = rtd->codec;
+	struct snd_soc_component *wm5102_component = asoc_rtd_to_codec(rtd, 0)->component;
 	int ret;
 	unsigned int old_params_set = priv->params_set;
 
@@ -773,7 +788,7 @@ static int rpi_cirrus_hw_free(struct snd_pcm_substream *substream)
 		dev_dbg(card->dev,
 			"hw_free: Setting SYSCLK to Zero\n");
 
-		ret = snd_soc_codec_set_sysclk(wm5102_codec,
+		ret = snd_soc_component_set_sysclk(wm5102_component,
 			ARIZONA_CLK_SYSCLK,
 			ARIZONA_CLK_SRC_FLL1,
 			0,
@@ -788,18 +803,18 @@ static int rpi_cirrus_hw_free(struct snd_pcm_substream *substream)
 
 static int rpi_cirrus_init_wm5102(struct snd_soc_pcm_runtime *rtd)
 {
-	struct snd_soc_codec *codec = rtd->codec;
+	struct snd_soc_component *component = asoc_rtd_to_codec(rtd, 0)->component;
 	int ret;
 
 	/* no 32kHz input, derive it from sysclk if needed  */
-	snd_soc_update_bits(codec,
+	snd_soc_component_update_bits(component,
 			ARIZONA_CLOCK_32K_1, ARIZONA_CLK_32K_SRC_MASK, 2);
 
-	if (rpi_cirrus_clear_flls(rtd->card, codec))
+	if (rpi_cirrus_clear_flls(rtd->card, component))
 		dev_warn(rtd->card->dev,
 			"init_wm5102: failed to clear FLLs\n");
 
-	ret = snd_soc_codec_set_sysclk(codec,
+	ret = snd_soc_component_set_sysclk(component,
 		ARIZONA_CLK_SYSCLK, ARIZONA_CLK_SRC_FLL1,
 		0, SND_SOC_CLOCK_IN);
 	if (ret) {
@@ -813,17 +828,18 @@ static int rpi_cirrus_init_wm5102(struct snd_soc_pcm_runtime *rtd)
 
 static int rpi_cirrus_init_wm8804(struct snd_soc_pcm_runtime *rtd)
 {
-	struct snd_soc_codec *codec = rtd->codec;
-	struct snd_soc_dai *codec_dai = rtd->codec_dai;
+	struct snd_soc_dai *codec_dai = asoc_rtd_to_codec(rtd, 0);
+	struct snd_soc_component *component = codec_dai->component;
 	struct snd_soc_card *card = rtd->card;
 	struct rpi_cirrus_priv *priv = snd_soc_card_get_drvdata(card);
-	unsigned int mask;
+	unsigned int val, mask;
 	int i, ret;
 
 	for (i = 0; i < 4; i++) {
+		val = snd_soc_component_read(component,
+			WM8804_SPDTX1 + i);
 		mask = (i == 3) ? 0x3f : 0xff;
-		priv->iec958_status[i] =
-			snd_soc_read(codec, WM8804_SPDTX1 + i) & mask;
+		priv->iec958_status[i] = val & mask;
 	}
 
 	/* Setup for 256fs */
@@ -861,30 +877,36 @@ static struct snd_soc_ops rpi_cirrus_ops = {
 	.hw_free = rpi_cirrus_hw_free,
 };
 
+SND_SOC_DAILINK_DEFS(wm5102,
+     DAILINK_COMP_ARRAY(COMP_EMPTY()),
+     DAILINK_COMP_ARRAY(COMP_CODEC("wm5102-codec", "wm5102-aif1")),
+     DAILINK_COMP_ARRAY(COMP_EMPTY()));
+
+SND_SOC_DAILINK_DEFS(wm8804,
+     DAILINK_COMP_ARRAY(COMP_CPU("wm5102-aif2")),
+     DAILINK_COMP_ARRAY(COMP_CODEC("wm8804.1-003b", "wm8804-spdif")));
+
 static struct snd_soc_dai_link rpi_cirrus_dai[] = {
 	[DAI_WM5102] = {
 		.name		= "WM5102",
 		.stream_name	= "WM5102 AiFi",
-		.codec_dai_name	= "wm5102-aif1",
-		.codec_name	= "wm5102-codec",
 		.dai_fmt	=   SND_SOC_DAIFMT_I2S
 				  | SND_SOC_DAIFMT_NB_NF
 				  | SND_SOC_DAIFMT_CBM_CFM,
 		.ops		= &rpi_cirrus_ops,
 		.init		= rpi_cirrus_init_wm5102,
+		SND_SOC_DAILINK_REG(wm5102),
 	},
 	[DAI_WM8804] = {
 		.name		= "WM5102 SPDIF",
 		.stream_name	= "SPDIF Tx/Rx",
-		.cpu_dai_name	= "wm5102-aif2",
-		.codec_dai_name	= "wm8804-spdif",
-		.codec_name	= "wm8804.1-003b",
 		.dai_fmt	=   SND_SOC_DAIFMT_I2S
 				  | SND_SOC_DAIFMT_NB_NF
 				  | SND_SOC_DAIFMT_CBM_CFM,
 		.ignore_suspend = 1,
 		.params		= &rpi_cirrus_dai_link2_params,
 		.init		= rpi_cirrus_init_wm8804,
+		SND_SOC_DAILINK_REG(wm8804),
 	},
 };
 
@@ -903,7 +925,7 @@ static int rpi_cirrus_late_probe(struct snd_soc_card *card)
 		priv->iec958_status[3]);
 
 	ret = snd_soc_dai_set_sysclk(
-		wm5102_runtime->codec_dai, ARIZONA_CLK_SYSCLK, 0, 0);
+		asoc_rtd_to_codec(wm5102_runtime, 0), ARIZONA_CLK_SYSCLK, 0, 0);
 	if (ret) {
 		dev_err(card->dev,
 			"Failed to set WM5102 codec dai clk domain: %d\n", ret);
@@ -911,7 +933,7 @@ static int rpi_cirrus_late_probe(struct snd_soc_card *card)
 	}
 
 	ret = snd_soc_dai_set_sysclk(
-		wm8804_runtime->cpu_dai, ARIZONA_CLK_SYSCLK, 0, 0);
+		asoc_rtd_to_cpu(wm8804_runtime, 0), ARIZONA_CLK_SYSCLK, 0, 0);
 	if (ret)
 		dev_err(card->dev,
 			"Failed to set WM8804 codec dai clk domain: %d\n", ret);
@@ -964,8 +986,8 @@ static int rpi_cirrus_probe(struct platform_device *pdev)
 		return -ENODEV;
 	}
 
-	rpi_cirrus_dai[DAI_WM5102].cpu_of_node = i2s_node;
-	rpi_cirrus_dai[DAI_WM5102].platform_of_node = i2s_node;
+	rpi_cirrus_dai[DAI_WM5102].cpus->of_node = i2s_node;
+	rpi_cirrus_dai[DAI_WM5102].platforms->of_node = i2s_node;
 
 	rpi_cirrus_card.dev = &pdev->dev;
 
